@@ -3,7 +3,7 @@
 #include "cwd.h"
 #include "chmod.h"
 #include "delete.h"
-#include "directorycache.h"
+#include "../directorycache.h"
 #include "directorylistingparser.h"
 #include "engineprivate.h"
 #include "externalipresolver.h"
@@ -565,7 +565,7 @@ void CFtpControlSocket::RawCommand(std::wstring const& command)
 	Push(std::make_unique<CFtpRawCommandOpData>(*this, command));
 }
 
-void CFtpControlSocket::Delete(CServerPath const& path, std::deque<std::wstring>&& files)
+void CFtpControlSocket::Delete(CServerPath const& path, std::vector<std::wstring>&& files)
 {
 	auto pData = std::make_unique<CFtpDeleteOpData>(*this);
 	pData->path_ = path;
