@@ -1,6 +1,6 @@
 #include <filezilla.h>
 
-#include "directorycache.h"
+#include "../directorycache.h"
 #include "pathcache.h"
 #include "rmd.h"
 
@@ -11,7 +11,7 @@ int CSftpRemoveDirOpData::Send()
 		fullPath = path_;
 
 		if (!fullPath.AddSegment(subDir_)) {
-			LogMessage(MessageType::Error, _("Path cannot be constructed for directory %s and subdir %s"), path_.GetPath(), subDir_);
+			log(logmsg::error, _("Path cannot be constructed for directory %s and subdir %s"), path_.GetPath(), subDir_);
 			return FZ_REPLY_ERROR;
 		}
 	}
@@ -32,12 +32,12 @@ int CSftpRemoveDirOpData::ParseResponse()
 	}
 
 	if (path_.empty()) {
-		LogMessage(MessageType::Debug_Info, L"Empty pData->path");
+		log(logmsg::debug_info, L"Empty pData->path");
 		return FZ_REPLY_INTERNALERROR;
 	}
 
 	engine_.GetDirectoryCache().RemoveDir(currentServer_, path_, subDir_, engine_.GetPathCache().Lookup(currentServer_, path_, subDir_));
-	controlSocket_.SendDirectoryListingNotification(path_, false, false);
+	controlSocket_.SendDirectoryListingNotification(path_, false);
 
 	return FZ_REPLY_OK;
 }
\ No newline at end of file
