@@ -10,6 +10,8 @@
 #include "verifycertdialog.h"
 #include "verifyhostkeydialog.h"
 
+#include <libfilezilla/translate.hpp>
+
 DECLARE_EVENT_TYPE(fzEVT_PROCESSASYNCREQUESTQUEUE, -1)
 DEFINE_EVENT_TYPE(fzEVT_PROCESSASYNCREQUESTQUEUE)
 
@@ -23,11 +25,13 @@ CAsyncRequestQueue::CAsyncRequestQueue(CMainFrame *pMainFrame)
 	, certStore_(std::make_unique<CertStore>())
 	, verifyCertDlg_(std::make_unique<CVerifyCertDialog>(*certStore_))
 {
+	CContextManager::Get()->RegisterHandler(this, STATECHANGE_REMOVECONTEXT, false);
 	m_timer.SetOwner(this);
 }
 
 CAsyncRequestQueue::~CAsyncRequestQueue()
 {
+	CContextManager::Get()->UnregisterHandler(this, STATECHANGE_REMOVECONTEXT);
 }
 
 bool CAsyncRequestQueue::ProcessDefaults(CFileZillaEngine *pEngine, std::unique_ptr<CAsyncRequestNotification> & pNotification)
@@ -93,20 +97,20 @@ bool CAsyncRequestQueue::ProcessDefaults(CFileZillaEngine *pEngine, std::unique_
 		{
 			auto & certNotification = static_cast<CCertificateNotification&>(*pNotification.get());
 
-			if (!certStore_->IsTrusted(certNotification)) {
+			if (!certStore_->IsTrusted(certNotification.info_)) {
 				break;
 			}
 
-			certNotification.m_trusted = true;
+			certNotification.trusted_ = true;
 			pEngine->SetAsyncRequestReply(std::move(pNotification));
 
 			return true;
 		}
 		break;
-	case reqId_insecure_ftp:
+	case reqId_insecure_connection:
 		{
-			auto & insecureNotification = static_cast<CInsecureFTPNotification&>(*pNotification.get());
-			if (!certStore_->IsInsecure(insecureNotification.server_.GetHost(), insecureNotification.server_.GetPort())) {
+			auto & insecureNotification = static_cast<CInsecureConnectionNotification&>(*pNotification.get());
+			if (!certStore_->IsInsecure(fz::to_utf8(insecureNotification.server_.GetHost()), insecureNotification.server_.GetPort())) {
 				break;
 			}
 
@@ -212,14 +216,14 @@ bool CAsyncRequestQueue::ProcessNextRequest()
 
 		SendReply(entry);
 	}
-	else if (entry.pNotification->GetRequestID() == reqId_insecure_ftp) {
+	else if (entry.pNotification->GetRequestID() == reqId_insecure_connection) {
 		if (!CheckWindowState()) {
 			return false;
 		}
 
-		auto & notification = static_cast<CInsecureFTPNotification&>(*entry.pNotification.get());
+		auto & notification = static_cast<CInsecureConnectionNotification&>(*entry.pNotification.get());
 
-		ConfirmInsecureConection(*certStore_.get(), notification);
+		ConfirmInsecureConection(m_pMainFrame, *certStore_.get(), notification);
 
 		SendReply(entry);
 	}
@@ -398,7 +402,7 @@ bool CAsyncRequestQueue::ProcessFileExistsNotification(t_queueEntry &entry)
 
 void CAsyncRequestQueue::ClearPending(CFileZillaEngine const* const pEngine)
 {
-	if (m_requestList.empty()) {
+	if (!pEngine) {
 		return;
 	}
 
@@ -512,3 +516,10 @@ bool CAsyncRequestQueue::SendReply(t_queueEntry& entry)
 
 	return entry.pEngine->SetAsyncRequestReply(std::move(entry.pNotification));
 }
+
+void CAsyncRequestQueue::OnStateChange(CState* pState, t_statechange_notifications notification, std::wstring const&, const void*)
+{
+	if (pState) {
+		ClearPending(pState->m_pEngine);
+	}
+}
